@@ -8,8 +8,14 @@ Provides rich, real-time visual feedback for CLI operations with:
 - Color coding (blue=active, green=success, red=error, yellow=info)
 - Non-blocking output with line replacement
 - Streaming text support
+- Text wrapping for clean output
+- Dynamic thinking messages for user engagement
 """
+import os
+import re
+import shutil
 import sys
+import textwrap
 import time
 import threading
 from typing import Optional, List, Callable, Any, Dict
@@ -17,6 +23,16 @@ from enum import Enum
 from dataclasses import dataclass, field
 from contextlib import contextmanager
 
+# Import dynamic thinking messages
+try:
+    from .thinking_messages import (
+        ThinkingPhase, get_thinking_message, get_tool_message,
+        get_progress_message, start_thinking, set_phase, should_update_message
+    )
+    HAS_THINKING_MESSAGES = True
+except ImportError:
+    HAS_THINKING_MESSAGES = False
+
 
 # =============================================================================
 # ANSI Color Codes
@@ -77,7 +93,7 @@ if not sys.stdout.isatty():
 # =============================================================================
 
 class Icons:
-    """Unicode icons for status display"""
+    """Unicode icons for status display - Claude Code style"""
     # Status indicators
     THINKING = "✻"
     SUCCESS = "✓"
@@ -85,11 +101,16 @@ class Icons:
     WARNING = "⚠"
     INFO = "ℹ"
 
+    # Claude Code style
+    BULLET = "●"           # Yellow bullet for tool calls
+    CORNER = "⎿"           # Corner for output indentation
+    HOLLOW = "○"           # Hollow circle for pending
+
     # Spinners
     DOTS = ["⠋", "⠙", "⠹", "⠸", "⠼", "⠴", "⠦", "⠧", "⠇", "⠏"]
     BRAILLE = ["⣾", "⣽", "⣻", "⢿", "⡿", "⣟", "⣯", "⣷"]
 
-    # Tree
+    # Tree (kept for backward compat)
     TREE_BRANCH = "└─"
     TREE_VERTICAL = "│"
     TREE_TEE = "├─"
@@ -169,7 +190,8 @@ class ActionSpinner:
         self,
         message: str = "Processing",
         spinner_chars: List[str] = None,
-        interval: float = 0.08
+        interval: float = 0.08,
+        dynamic_messages: bool = True
     ):
         """Initialize the action spinner.
 
@@ -177,10 +199,13 @@ class ActionSpinner:
             message: Initial status message
             spinner_chars: Characters for spinner animation
             interval: Animation interval in seconds
+            dynamic_messages: Whether to use dynamic thinking messages
         """
         self.message = message
+        self.initial_message = message
         self.spinner_chars = spinner_chars or Icons.DOTS
         self.interval = interval
+        self.dynamic_messages = dynamic_messages and HAS_THINKING_MESSAGES
 
         self.running = False
         self.frame_index = 0
@@ -190,10 +215,16 @@ class ActionSpinner:
         self._thread: Optional[threading.Thread] = None
         self._lock = threading.Lock()
         self._last_render_lines = 0
+        self._start_time: Optional[float] = None
+        self._last_message_update: float = 0
+        self._message_update_interval: float = 4.0  # Update message every 4 seconds
 
     def start(self) -> "ActionSpinner":
         """Start the spinner animation."""
         self.running = True
+        self._start_time = time.time()
+        if self.dynamic_messages:
+            start_thinking()
         sys.stdout.write(Color.HIDE_CURSOR)
         sys.stdout.flush()
         self._thread = threading.Thread(target=self._animate, daemon=True)
@@ -211,6 +242,13 @@ class ActionSpinner:
     def _animate(self) -> None:
         """Animation loop running in background thread."""
         while self.running:
+            # Update message dynamically if enabled
+            if self.dynamic_messages and self._start_time:
+                now = time.time()
+                if now - self._last_message_update >= self._message_update_interval:
+                    self._last_message_update = now
+                    self.message = get_progress_message()
+
             self._render()
             self.frame_index = (self.frame_index + 1) % len(self.spinner_chars)
             time.sleep(self.interval)
@@ -510,8 +548,17 @@ def status(message: str, state: str = "info") -> None:
     print(f"{icon} {message}")
 
 
-def thinking(message: str) -> None:
-    """Print a thinking/processing status."""
+def thinking(message: str = None, dynamic: bool = True) -> None:
+    """Print a thinking/processing status.
+
+    Args:
+        message: Custom message (optional, uses dynamic message if None)
+        dynamic: Whether to use dynamic thinking messages
+    """
+    if message is None and dynamic and HAS_THINKING_MESSAGES:
+        message = get_thinking_message()
+    elif message is None:
+        message = "Thinking..."
     status(message, "thinking")
 
 
@@ -536,25 +583,67 @@ def info(message: str) -> None:
 
 
 # =============================================================================
-# Action Logging
+# Action Logging - Claude Code Style
 # =============================================================================
 
 def log_action(action: str, target: str, state: str = "running") -> None:
-    """Log a tool/action with its target.
+    """Log a tool/action with its target in Claude Code style.
+
+    Format: ● Action(target)
 
     Args:
         action: Action name (Read, Write, Bash, etc.)
         target: Action target (filename, command, etc.)
         state: Action state (running, success, error)
     """
-    states = {
-        "running": f"{Color.BLUE}●{Color.RESET}",
-        "success": f"{Color.GREEN}{Icons.SUCCESS}{Color.RESET}",
-        "error": f"{Color.RED}{Icons.FAILURE}{Color.RESET}",
-        "pending": f"{Color.DIM}○{Color.RESET}",
-    }
-    icon = states.get(state, states["running"])
-    print(f"  {Icons.TREE_BRANCH} {icon} {action}({Color.CYAN}{target}{Color.RESET})")
+    # Yellow bullet for all tool calls (Claude Code style)
+    bullet = f"{Color.YELLOW}{Icons.BULLET}{Color.RESET}"
+    print(f"{bullet} {Color.BOLD}{action}{Color.RESET}({Color.CYAN}{target}{Color.RESET})")
+
+
+def log_output(output: str, is_error: bool = False, max_lines: int = 10) -> None:
+    """Log tool output with Claude Code style corner indentation.
+
+    Format:
+      ⎿  Output line 1
+         Output line 2
+
+    Args:
+        output: Output text to display
+        is_error: Whether this is error output
+        max_lines: Max lines to show before truncating (0 = no limit)
+    """
+    if not output or not output.strip():
+        return
+
+    lines = output.strip().split('\n')
+    show_truncation = max_lines > 0 and len(lines) > max_lines
+
+    if show_truncation:
+        display_lines = lines[:max_lines]
+    else:
+        display_lines = lines
+
+    # Color for output
+    text_color = Color.RED if is_error else Color.DIM
+
+    for i, line in enumerate(display_lines):
+        if i == 0:
+            # First line gets the corner symbol
+            prefix = f"  {Color.DIM}{Icons.CORNER}{Color.RESET}  "
+        else:
+            # Subsequent lines get spacing to align
+            prefix = "     "
+
+        if is_error and i == 0:
+            # Error prefix for first line
+            print(f"{prefix}{Color.RED}Error:{Color.RESET} {text_color}{line}{Color.RESET}")
+        else:
+            print(f"{prefix}{text_color}{line}{Color.RESET}")
+
+    if show_truncation:
+        remaining = len(lines) - max_lines
+        print(f"     {Color.DIM}... (+{remaining} more lines){Color.RESET}")
 
 
 # =============================================================================
@@ -809,32 +898,225 @@ class TaskDisplay:
         print(f"{Color.DIM}   {success_count} steps completed in {duration:.1f}s{Color.RESET}\n")
 
 
+# =============================================================================
+# Text Wrapping and Response Formatting
+# =============================================================================
+
+def get_terminal_width() -> int:
+    """Get terminal width, with fallback to 80 columns.
+
+    Returns:
+        Terminal width in columns
+    """
+    try:
+        size = shutil.get_terminal_size()
+        return size.columns
+    except Exception:
+        return 80
+
+
+def get_response_width(percentage: float = 0.75) -> int:
+    """Get the width for response output (default 75% of terminal).
+
+    Args:
+        percentage: Fraction of terminal width to use (0.0 to 1.0)
+
+    Returns:
+        Width in columns for response text
+    """
+    terminal_width = get_terminal_width()
+    width = int(terminal_width * percentage)
+    # Ensure minimum width of 40 and max of terminal width
+    return max(40, min(width, terminal_width))
+
+
+def wrap_text(text: str, width: Optional[int] = None, indent: str = "") -> str:
+    """Wrap text to specified width, preserving paragraphs and code blocks.
+
+    Args:
+        text: Text to wrap
+        width: Maximum width (default: 75% of terminal)
+        indent: Prefix for each line
+
+    Returns:
+        Wrapped text string
+    """
+    if width is None:
+        width = get_response_width()
+
+    # Adjust width for indent
+    effective_width = width - len(indent)
+    if effective_width < 20:
+        effective_width = 20
+
+    lines = []
+    in_code_block = False
+    code_block_content = []
+
+    for line in text.split('\n'):
+        # Check for code block markers
+        if line.strip().startswith('```'):
+            if in_code_block:
+                # End of code block - add as-is
+                code_block_content.append(line)
+                lines.extend(code_block_content)
+                code_block_content = []
+                in_code_block = False
+            else:
+                # Start of code block
+                in_code_block = True
+                code_block_content = [line]
+            continue
+
+        if in_code_block:
+            # Inside code block - don't wrap
+            code_block_content.append(line)
+            continue
+
+        # Empty line - preserve paragraph break
+        if not line.strip():
+            lines.append("")
+            continue
+
+        # Check for list items or special formatting to preserve
+        stripped = line.lstrip()
+        leading_spaces = len(line) - len(stripped)
+
+        # Preserve list items (- or * or numbered)
+        if stripped.startswith(('-', '*', '•')) or re.match(r'^\d+\.', stripped):
+            # Wrap list items with hanging indent
+            list_indent = ' ' * leading_spaces
+            # Find the marker and content
+            match = re.match(r'^([-*•]|\d+\.)\s*', stripped)
+            if match:
+                marker = match.group(0)
+                content = stripped[len(marker):]
+                subsequent_indent = list_indent + ' ' * len(marker)
+                wrapped = textwrap.fill(
+                    content,
+                    width=effective_width,
+                    initial_indent=list_indent + marker,
+                    subsequent_indent=subsequent_indent
+                )
+                lines.append(wrapped)
+            else:
+                lines.append(line)
+            continue
+
+        # Check for headers (## style)
+        if stripped.startswith('#'):
+            lines.append(line)
+            continue
+
+        # Regular paragraph - wrap with preserved leading indent
+        if leading_spaces > 0:
+            para_indent = ' ' * leading_spaces
+            wrapped = textwrap.fill(
+                stripped,
+                width=effective_width,
+                initial_indent=para_indent,
+                subsequent_indent=para_indent
+            )
+        else:
+            wrapped = textwrap.fill(stripped, width=effective_width)
+
+        lines.append(wrapped)
+
+    # Handle unclosed code block
+    if in_code_block:
+        lines.extend(code_block_content)
+
+    # Apply indent to all lines
+    if indent:
+        lines = [indent + line if line else line for line in lines]
+
+    return '\n'.join(lines)
+
+
+def format_response(response: str, width_percentage: float = 0.75) -> str:
+    """Format an AI response for clean terminal display.
+
+    This wraps text to 75% of terminal width (configurable) while:
+    - Preserving code blocks (``` ... ```)
+    - Preserving list formatting
+    - Preserving headers
+    - Maintaining paragraph breaks
+
+    Args:
+        response: The response text to format
+        width_percentage: Fraction of terminal width to use
+
+    Returns:
+        Formatted response string
+    """
+    if not response:
+        return response
+
+    width = get_response_width(width_percentage)
+    return wrap_text(response, width)
+
+
+def print_response(response: str, width_percentage: float = 0.75) -> None:
+    """Print a formatted response with proper text wrapping.
+
+    The agent's response is visually distinct from user input with:
+    - A colored "NC1709" header
+    - Cyan-colored text
+    - Left border indicator
+
+    Args:
+        response: Response text to print
+        width_percentage: Fraction of terminal width to use
+    """
+    # Account for border prefix "│ " (2 chars) in width calculation
+    # Reduce percentage slightly to ensure text + border fits within 75% of terminal
+    border_width = 2
+    terminal_width = get_terminal_width()
+    effective_percentage = (width_percentage * terminal_width - border_width) / terminal_width
+
+    formatted = format_response(response, effective_percentage)
+
+    # Add visual distinction for agent responses
+    # Header with robot icon and name
+    header = f"\n{Color.BOLD}{Color.CYAN}◆ NC1709{Color.RESET}"
+
+    # Add subtle left border by indenting and coloring
+    lines = formatted.split('\n')
+    bordered_lines = []
+    for line in lines:
+        # Use dim cyan vertical bar as left border + agent text in cyan
+        bordered_lines.append(f"{Color.DIM}{Color.CYAN}│{Color.RESET} {Color.CYAN}{line}{Color.RESET}")
+
+    bordered_response = '\n'.join(bordered_lines)
+
+    print(header)
+    print(bordered_response)
+    print()  # Extra line for spacing
+
+
 # =============================================================================
 # Demo / Test
 # =============================================================================
 
 def demo():
-    """Demonstrate CLI UI components."""
+    """Demonstrate CLI UI components - Claude Code style."""
     print("\n" + "=" * 60)
-    print("NC1709 CLI UI Demo")
+    print("NC1709 CLI UI Demo - Claude Code Style")
     print("=" * 60 + "\n")
 
-    # Demo 1: Simple spinner
-    print("Demo 1: Action Spinner")
+    # Demo 1: Claude Code style tool calls
+    print("Demo 1: Tool Calls (Claude Code Style)")
     print("-" * 40)
 
-    with action_spinner("Analyzing your request") as spinner:
-        time.sleep(0.5)
-        spinner.update("Planning implementation steps")
-        time.sleep(0.5)
-        idx = spinner.add_action("Read", "main.py")
-        time.sleep(0.3)
-        spinner.complete_action(idx)
-        idx = spinner.add_action("Write", "feature.py")
-        time.sleep(0.3)
-        spinner.complete_action(idx)
-        time.sleep(0.2)
-        spinner.success("Implementation complete")
+    # Simulate tool calls with output
+    log_action("Read", "main.py")
+    log_output("def main():\n    print('Hello World')\n\nif __name__ == '__main__':\n    main()")
+
+    log_action("Bash", "python main.py")
+    log_output("Hello World")
+
+    log_action("Bash", "python broken.py")
+    log_output("ModuleNotFoundError: No module named 'nonexistent'", is_error=True)
 
     print()
 
@@ -848,29 +1130,33 @@ def demo():
     error("Failed to parse config.json")
     print()
 
-    # Demo 3: Task display
-    print("Demo 3: Task Display")
+    # Demo 3: Multi-line output with truncation
+    print("Demo 3: Multi-line Output")
     print("-" * 40)
+    log_action("Grep", "TODO")
+    long_output = "\n".join([f"src/file{i}.py:42: # TODO: implement feature {i}" for i in range(15)])
+    log_output(long_output, max_lines=5)
 
-    task = TaskDisplay("Implementing authentication feature")
-    task.start()
+    print()
 
-    task.step("Analyzing requirements")
-    idx = task.action("Read", "auth_spec.md")
-    time.sleep(0.3)
-    task.complete_action(idx)
-    task.complete_step("Requirements analyzed")
+    # Demo 4: Response formatting
+    print("Demo 4: Agent Response")
+    print("-" * 40)
+    sample_response = """I've analyzed your codebase and found the following:
 
-    task.step("Creating auth module")
-    idx1 = task.action("Write", "auth/login.py")
-    time.sleep(0.2)
-    task.complete_action(idx1)
-    idx2 = task.action("Write", "auth/logout.py")
-    time.sleep(0.2)
-    task.complete_action(idx2)
-    task.complete_step("Auth module created")
+1. The main entry point is in `main.py`
+2. Configuration is loaded from `config.json`
+3. There are 3 utility modules in the `utils/` directory
 
-    task.finish()
+Here's a quick example of how to run it:
+
+```python
+from myapp import main
+main.run()
+```
+
+Let me know if you need any additional help!"""
+    print_response(sample_response)
 
     print("Demo complete!")
 
